(import '(java.util Properties) '(java.lang Process System)
        '(java.awt.event ActionListener ItemListener)
        '(java.awt GridLayout GridBagLayout GridBagConstraints)
	'(java.io FileInputStream BufferedReader InputStreamReader FileWriter)
	'(javax.swing JFrame JLabel JTextField JButton JComboBox 
		      JSpinner SpinnerNumberModel JOptionPane) )

(defn cmd [p]
  (.. Runtime getRuntime (exec (str p))))
(defn cmdout [o]
  (let [r (BufferedReader. (InputStreamReader. (.getInputStream o)))]
    (dorun (map println (line-seq r)))))

(defn header-str [cdf-name]
  (str "#header\nCDF NAME: " cdf-name "\nDATA ENCODING: NETWORK\n"
       "MAJORITY: ROW\n0/0 99 0 0/z 0\n#GLOBALattributes\n"))
(defn attr-str [name value]
  (if (= "" (.trim (str value)))
    nil
    (str "'" name "' 1: CDF_CHAR { '" value "' } .\n")))
(defn footer-str []
  (str "#VARIABLEattributes\n#variables\n#end\n"))

(defn add-items [model items]
  (doseq [elem items] (.addItem model elem)))

(defn label [text]
  (JLabel. text JTextField/RIGHT))

(defn cell [x y]
  (let [gbc (GridBagConstraints.)]
    (set! (.gridx gbc) x) (set! (.gridy gbc) y) (set! (.ipady gbc) 5)
    (set! (.fill gbc) GridBagConstraints/HORIZONTAL)
    gbc))

(defn bag-row [frame row text widget]
  (.add frame (label text) (cell 0 row))
  (.add frame widget (cell 1 row)))

(defn error-dialog [frame message]
  (JOptionPane/showMessageDialog 
   frame message "CDAWeb Attributes" JOptionPane/ERROR_MESSAGE))

(defn cdaw []
  (let [frame (JFrame. "CDAWeb Attributes")
	project-widget (JTextField.)
	source-name-widget (JComboBox.)
	source-desc-widget (JTextField.)
	data-type-widget (JComboBox.)
	pi-name-widget (JTextField.)
	pi-org-widget (JTextField.)
	discipline-widget (JComboBox.)
	mission-widget (JComboBox.)
	instrument-widget (JComboBox.)
	version-widget (JSpinner. (SpinnerNumberModel. 1 1 99 1))
	time-range-widget (JTextField.)
	emit-button (JButton. "Emit") 
	props (Properties.)]

    (. props (load (FileInputStream. (System/getProperty "props"))))
    (doto source-name-widget
      (add-items (.split (. props (getProperty "gAttr.source_names")) ":")))
    (doto data-type-widget
      (add-items (.split (. props (getProperty "gAttr.data_types")) ":")))
    (doto discipline-widget
      (.setEditable true)
      (add-items (.split (. props (getProperty "gAttr.disciplines")) ":")))
    (doto mission-widget
      (add-items (.split (. props (getProperty "gAttr.missionGroups")) ":")))
    (doto instrument-widget
      (add-items (.split (. props (getProperty "gAttr.instrumentTypes")) ":")))

    (.addActionListener emit-button
      (proxy [ActionListener] []
        (actionPerformed [evt]
	  (let [skt-list (list
	    (header-str "/dev/null")
	    (attr-str "Project" (.getText project-widget))
	    (attr-str "Source_name" (.getSelectedItem source-name-widget))
	    (attr-str "Logical_source_description"(.getText source-desc-widget))
	    (attr-str "Data_type" (.getSelectedItem data-type-widget))
	    (attr-str "PI_name" (.getText pi-name-widget))
	    (attr-str "PI_affiliation" (.getText pi-org-widget))
	    (attr-str "Discipline" (.getSelectedItem discipline-widget))
	    (attr-str "Mission_group" (.getSelectedItem mission-widget))
	    (attr-str "Instrument_type" (.getSelectedItem instrument-widget))
	    (attr-str "Data_version" (.getValue (.getModel version-widget)))
	    (attr-str "Descriptor" (.getText time-range-widget))
	    (footer-str) )]

	    (if (some nil? skt-list)
	      (error-dialog frame "No blank fields allowed")
	      (let [skt-str (apply str skt-list)]
		(with-open [skt-file (FileWriter. "t.skt")]
		  (.write skt-file skt-str) )
		(println skt-str)
		(let [p (cmd "skeletoncdf -delete -cdf t.cdf t.skt")]
		  (cmdout p)
		  (if (not (zero? (.waitFor p)))
		    (error-dialog frame "skeletoncdf error"))))) ))))

    (doto frame
	  (.setLayout (GridBagLayout.))
	  (.setDefaultCloseOperation JFrame/DISPOSE_ON_CLOSE)
	  (bag-row  0 "Project: " project-widget)
	  (bag-row  1 "Source Name: " source-name-widget)
	  (bag-row  2 "Source Description: " source-desc-widget)
	  (bag-row  3 "Data Type: " data-type-widget)
	  (bag-row  4 "PI Name: " pi-name-widget)
	  (bag-row  5 "PI Affiliation: " pi-org-widget)
	  (bag-row  6 "Discipline: " discipline-widget)
	  (bag-row  7 "Mission: " mission-widget)
	  (bag-row  8 "Instrument: " instrument-widget)
	  (bag-row  9 "Version: " version-widget)
	  (bag-row 10 "Time Range: " time-range-widget)
	  (.add emit-button (cell 0 11))
	  (.pack) ;; (.setSize 800 450)
	  (.setVisible true))))
(cdaw)
