// utils.c -- a few handy utility routines.

#include "utils.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static void
put_args(int err, const char *type, const char *fmt, va_list args)
{
  fflush(stdout);
  if (program_name != NULL)
    fprintf(stderr, "%s: ", program_name);
  fprintf(stderr, "%s: ", type);
  vfprintf(stderr, fmt, args);
  if (err > 0)
    fprintf(stderr, ": %s", strerror(err));
  else if (err < 0)
    {
      char cdf_msg[CDF_STATUSTEXT_LEN + 1];
      CDFgetStatusText((long)err, cdf_msg);
      fprintf(stderr, ": %s", cdf_msg);
    }
  fprintf(stderr, "\n");
  fflush(stderr);
}

void
debug(int level, const char *fmt, ...)
{
  va_list args;
  va_start(args, fmt);
  if (level <= debug_level)
    put_args(0, "debug", fmt, args);
  va_end(args);
}

void
warn(int err, const char *fmt, ...)
{
  va_list args;
  va_start(args, fmt);
  put_args(err, "WARN", fmt, args);
  va_end(args);
}

void
fail(int err, const char *fmt, ...)
{
  va_list args;
  va_start(args, fmt);
  put_args(err, "FAIL", fmt, args);
  va_end(args);
  exit(EXIT_FAILURE);
}

void
cdf_fail_unless(CDFstatus status, const char *fmt, ...)
{
  if (status != CDF_OK)
    {
      va_list args;
      va_start(args, fmt);
      put_args(status, "FAIL", fmt, args);
      va_end(args);
      exit(EXIT_FAILURE);
    }
}

int
streq(const char *s1, const char *s2)
{
  return strcmp(s1, s2) == 0;
}
